#!/usr/bin/perl
#
# dumpfw.pl - Make a debugging dump of a compound firmware binary
#
# Written 2005, 2006 by Werner Almesberger
# Copyright 2005, 2006 by Werner Almesberger
#

print STDERR "usage: $0 firmware.bin compound.bin\n" unless @ARGV == 2;

open(FILE,$ARGV[0]) || die "$ARGV[0]: $!";
$fw = join("",<FILE>);
close FILE;

open(FILE,$ARGV[1]) || die "$ARGV[1]: $!";
$bin = join("",<FILE>);
close FILE;

$p = length $fw;
die "$_[1] was not made from $_[0]" if substr($bin,0,$p) ne $fw;

print "table\n";
while (1) {
    $obj = unpack("v",substr($bin,$p,2));
    $p += 2;
    if (!$obj) {
	print "  (end)\n";
	last;
    }
    print sprintf("  0x%04x\n",$obj);
}

while ($p < length $bin) {
    ($width,$flyback) = unpack("CC",substr($bin,$p,2));
    $p += 2;
    if ($width) {
	print sprintf("0x%04x: image (width $width, flyback %d)",
	  ($p+0x1fe)/2,$flyback*16);
	for ($i = 0; $i != $width; $i++) {
	    print "\n " if !($i % 6);
	    ($a0,$b0,$a1,$b1) = unpack("CCCC",substr($bin,$p,4));
	    $p += 4;
	    print sprintf(" %02x.%02x.%02x.%02x",$a0,$b0,$a1,$b1);
	}
	print "\n";
    }
    else {
	print sprintf("0x%04x: sequence\n",($p+0x1fe)/2);
	$obj = unpack("v",substr($bin,$p,2));
	$p += 2;
	print sprintf("  title 0x%04x\n",$obj);
	while (1) {
	    ($rot,$flags,$to,$obj) = unpack("CCvv",substr($bin,$p,6));
	    $p += 6;
	    print sprintf("    obj 0x%04x, rotate $rot, timeout $to%s\n",
	      $obj,$flags ? $flags == 1 ? "\n    end" : "\n    loop" : "");
	    last if $flags;
	}
    }
}
