;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Capacitive sensor sensor experiment (ADC)
;

;
; A capacitive sensor is connected between RA1/AN1 and VDD, with a 1-3M
; parallel resistor. We first charge the capacitor, then measure the voltage
; across it after a certain delay.
;

;
; The analog signal should be in the range 0-1.7V with Vcc = 3V, so we use a
; bit more than half the ADC range. The output routine below should yield a
; value in the range [0,18]. We're extra-careful and limit it to NUM_LEDS-1,
; just in case.
;

	cblock
lastcap
	endc

cap_adc_xp:
;	call	button		; button pressed ?
;	btfss	STATUS,C
;	goto	ui_off		; no -> power down

	; prepare RA1/AN1 for analog input and set up the ADC

	bsf	STATUS,RP0
	bcf	TRISA,1		; enable digital output
	movlw	1 << 1		; enable analog input
	movwf	ANSEL
	movlw	1 << ADFM	; right justified, rail-to-rail
	movwf	ADCON1
	bcf	STATUS,RP0

	movlw	(1 << ADCS0) | (1 << CHS0) | (1 << ADON)
				; Fosc/8, select AN1, ADC on
	movwf	ADCON0

	; load the capacitor

	call	lightsout

	movlw	2
	call	waitw

	bcf	PORTA,1		; a full charge takes less than 1 us at 1 kOhm

	; start the ramp

	bsf	STATUS,RP0
	bsf	TRISA,1
	bcf	STATUS,RP0

	movlw	4		; wait 278 us
	call	waitw

	bsf	ADCON0,GO	; start the conversion
caxalp:
	btfsc	ADCON0,NOT_DONE	; wait until done
	goto 	caxalp

	bsf	STATUS,RP0	; enable digital output again
	bcf	TRISA,1
	clrf	ANSEL		; disable analog input
	movf	ADRESL,W	; get the result
	bcf	STATUS,RP0
	clrf	ADCON0		; turn off the ADC

	subwf	lastcap,W	; W = lastcap-res
	subwf	lastcap,F	; lastcap = lastcap-(lastcap-res) = res
	; show delta around center: led = (12-d)-23 = -11-d
	addlw	-(NUM_LEDS/2-1)	; out of [0,NUM_LEDS-1] -> ignore
	addlw	NUM_LEDS-1
	btfsc	STATUS,C
	call	setwth		; show what we got

	movlw	14		; hold display for 9 ms
	call	waitw

	goto	cap_adc_xp
