;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Illumination sensor experiment (ADC)
;

;
; The analog signal should be in the range 0-1.7V with Vcc = 3V, so we use a
; bit more than half the ADC range. The output routine below should yield a
; value in the range [0,18]. We're extra-careful and limit it to NUM_LEDS-1,
; just in case.
;


ill_adc_xp:
	call	button		; button pressed ?
	btfss	STATUS,C
	goto	ui_off		; no -> power down

	; prepare AN2 for analog input

	bsf	STATUS,RP0
	bsf	TRISA,2		; turn off digital output
	movlw	1 << 2		; enable analog input
	movwf	ANSEL
	clrf	ADCON1		; left justified
	bcf	STATUS,RP0

	movlw	(1 << ADCS0) | (2 << CHS0) | (1 << ADON)
				; Fosc/8, select AN2, ADC on
	movwf	ADCON0

	; power up the sensor

if (HW_SINK)
	call	lightsout
else
	clrf	PORTA
	clrf	PORTB
	bsf	PORTA,3
endif

	movlw	5		; wait 501 us
	call	waitw

	bsf	ADCON0,GO	; start the conversion
adclp:
	btfsc	ADCON0,NOT_DONE	; wait until done
	goto 	adclp

	bcf	PORTA,3		; power down the sensor
	bsf	STATUS,RP0	; enable digital output again
	bcf	TRISA,2
	clrf	ANSEL		; disable analog input
	bcf	STATUS,RP0
	clrf	ADCON0		; turn off the ADC

	rrf	ADRESH,F	; get upper 5 bits
	rrf	ADRESH,F
	rrf	ADRESH,F
	movlw	0x1f
	andwf	ADRESH,F
	movlw	-(NUM_LEDS-1)	; cut off range to [0,NUM_LEDS-1]
	addwf	ADRESH,W
	movlw	NUM_LEDS-1
	btfss	STATUS,C
	movf	ADRESH,W

	call	setwth
	goto	ill_adc_xp
